/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "new.h"

// This is the default constructor for the new project dialog. It is meant
// to initialize all of the objects and set their properties. It then creates
// the dialog and displays it to the screen
NewProjectDialog::NewProjectDialog (TemplateList *list_of_templates)
{
  tlist = list_of_templates;
  initialize_objects();
  populate_combobox();

  location->set_text(Glib::ustring(Glib::get_home_dir() + "/openldev/"));
  apply->signal_clicked().connect(sigc::mem_fun(*this, &NewProjectDialog::on_apply));
  cancel->signal_clicked().connect(sigc::mem_fun(*this, &NewProjectDialog::on_cancel));
  browse->signal_clicked().connect(sigc::mem_fun(*this, &NewProjectDialog::on_browse));
}

// The dialog and its widgets are initialized in this function.
void NewProjectDialog::initialize_objects()
{
  Glib::ustring gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/newproject.glade";
  Glib::RefPtr<Gnome::Glade::Xml> xml = Gnome::Glade::Xml::create(gladefile);
  
  xml->get_widget("dialog", dialog);
  xml->get_widget("comboboxentry", combobox);
  xml->get_widget("name", name);
  xml->get_widget("version", version);
  xml->get_widget("homepage", homepage);
  xml->get_widget("email", email);
  xml->get_widget("author", author);
  xml->get_widget("location", location);
  xml->get_widget("button_cancel", cancel);
  xml->get_widget("button_ok", apply);
  xml->get_widget("button_browse", browse);
}

// This destructor goes through all of the initialized widgets
// and deletes them to free up memory on close of the dialog
NewProjectDialog::~NewProjectDialog()
{
  delete cancel;
  delete apply;
  delete browse;
  delete name;
  delete author;
  delete email;
  delete homepage;
  delete version;
  delete location;
  delete combobox;
}

Gtk::Dialog* NewProjectDialog::get_dialog()
{ return dialog;
}

// When cancel is pressed, this function is called to emit a cancel signal
// and hide the dialog
void NewProjectDialog::on_cancel(void)
{
  dialog->response(Gtk::RESPONSE_CANCEL);
  dialog->hide();
}

// When the browse button is pressed, the user is presented with a 
// FileChooserDialog where they can choose where to create the project.
void NewProjectDialog::on_browse(void)
{
  Gtk::FileChooserDialog filedialog("Choose Directory", Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
  filedialog.set_transient_for(*dialog);
  filedialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  filedialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  
  int result = filedialog.run();
  switch (result)
  {
    // If OK is pressed
    case (Gtk::RESPONSE_OK):
    {
      location->set_text(Glib::ustring(filedialog.get_filename() + "/"));
      break;
    }
    default:
      break;
  }
}

// Check the dialog for the following errors: If the project name is empty or
// contains non-alphanumeric characters, if the version is empty, if the project
// locatino is empty
Glib::ustring NewProjectDialog::check_for_errors()
{
  Glib::ustring ntxt = name->get_text(), vtxt = version->get_text(),
                ltxt = location->get_text();
  
  // Check for error when creating a project or file
  if (ntxt.find(" ",0) != Glib::ustring::npos ||
      ntxt.find("_",0) != Glib::ustring::npos ||
      ntxt.empty())
    return "The project name must\nbe alphanumeric!";
  else if (vtxt.empty())
    return "You must specify a\nversion number!";
  else if (ltxt.empty())
    return "You must specify a\nproject location!";
  else
    return "NONE";
}

// Create the directory the project is to be created in
void NewProjectDialog::create_location()
{
  Glib::ustring str = location->get_text();
  mkdir(str.c_str(), 0777);
  str += name->get_text() + string("/");
  mkdir(str.c_str(), 0777);
  str += "src/";
  mkdir(str.c_str(), 0777);
}

// Create a new project file. This function returns text that is to be added
// to the project file and returns by reference the location the project file
// is to be created.
Glib::ustring NewProjectDialog::create_odp_file_text(Glib::ustring &dir)
{
  Gtk::Entry *choice = combobox->get_entry();
  Glib::ustring odp_file_text, choice_text = choice->get_text(), sources;
  
  if (choice_text.find("C++", 0) != string::npos)
    sources = "main.cc/";
  else
    sources = "main.c/";
    
  
  // Create the text for the file
  odp_file_text = "<name>" + name->get_text() + "*\n<author>" + author->get_text()
    + "*\n<location>" + location->get_text() + name->get_text()
    + "/*\n<email>" + email->get_text() + "*\n<url>" + homepage->get_text() 
    + "*\n<version>" + version->get_text();
  
  // Add the files that need to be added for the template
  for (unsigned int i=0; i < tlist->title.size(); i++)
  {
    if (tlist->title.at(i) == choice_text)
    {
      odp_file_text += string("*\n<template>") + tlist->file.at(i) 
                    + "*\n<lastfile>" + sources.substr(0, sources.find("/", 0))
                    + "*\n<sources>" + sources
                    + "*\n<headers>*\n<otherfiles>*\n<execname>";
    }
  }
    
  dir = location->get_text() + name->get_text() + "/" + name->get_text() + ".openldev";
  
  return odp_file_text;
}

void NewProjectDialog::clean_white_space_and_replace_content (Glib::ustring &text, bool isMakefile)
{
  Glib::ustring firststring[] = { "AUTHOR", "HOMEPAGE", "EMAIL", "TITLE", "VERSION", "[[", "]]", "AMP" };
  Glib::ustring replacement[] = { author->get_text(), homepage->get_text(),
                email->get_text(), name->get_text(), version->get_text(), "<", ">", "&" };
  
  for (unsigned int i=0; i<8; i++)
    while ((text.find(firststring[i], 0) != string::npos))
    {
      if (!(i == 0 && isMakefile))
      {
        int l = text.find(firststring[i], 0);
        text.erase(l, firststring[i].length());
        text.insert(l, replacement[i]);
      }
      else
        break;
    }
}

void NewProjectDialog::create_project_from_template (Glib::ustring filename)
{
  Glib::ustring file = string(INSTALL_PREFIX) + "/share/openldev/templates/"
                     + filename + ".template";
  xmlDocPtr doc = xmlParseFile (file.c_str());
  xmlNodePtr cur;
  
  if (doc != NULL) 
  {
    cur = xmlDocGetRootElement(doc);
    if (!xmlStrcmp(cur->name, (const xmlChar *) "project"))
    {
      cur = cur->xmlChildrenNode;
      while (cur != NULL)
      {
        if (!xmlStrcmp(cur->name, (const xmlChar *) "basefiles"))
        {
          cur = cur->xmlChildrenNode;
          while (cur != NULL)
          {
            if (!xmlStrcmp(cur->name, (const xmlChar *) "file"))
            {
              Glib::ustring fn = location->get_text() + name->get_text() + "/" 
                               + (const char *)xmlGetProp(cur, (const xmlChar *) "name");
              Glib::ustring content = (const char *)xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
              
              if (content.find("AUTHORS", 0) != string::npos)
                clean_white_space_and_replace_content(content, true);
              else
                clean_white_space_and_replace_content(content, false);
              write_file(fn, content);
              
              if (content.find("#!/bin/sh", 0) != string::npos)
                chmod(fn.c_str(), 0777);
            }
            
            cur = cur->next;
          }

          break;
        }
        cur = cur->next;
      }
      
      cur = xmlDocGetRootElement(doc);
      cur = cur->xmlChildrenNode;
      while (cur != NULL)
      {
        if (!xmlStrcmp(cur->name, (const xmlChar *) "srcfiles"))
        {
          cur = cur->xmlChildrenNode;
          while (cur != NULL)
          {
            if (!xmlStrcmp(cur->name, (const xmlChar *) "file"))
            {
              Glib::ustring fn = location->get_text() + Glib::ustring(name->get_text()) + "/src/" 
                      + (const char *)xmlGetProp(cur, (const xmlChar *) "name");
              Glib::ustring content = (const char *) xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
              clean_white_space_and_replace_content(content, false);
              write_file(fn, content);
              if (content.find("#!/bin/sh", 0) != string::npos)
                chmod(fn.c_str(), 0777);
            }
            
            cur = cur->next;
          }
          
          break;
        }
        cur = cur->next;
      }
    }
  }
  
  xmlFreeDoc(doc);
}

// When the apply button is pressed, it is checked that the project name only
// contains letters and numbers and that a version and location is specified.
// The project is then created from a template and the Makefiles and configure
// file are edited. The OpenLDev project file is also created.
void NewProjectDialog::on_apply()
{
  Glib::ustring errortext = check_for_errors();
  
  // Create new project
  if (errortext == "NONE")
  {
    Glib::ustring odp_file_text, odp_file_location, project_path, copy, extract,
                  rm, configure_in, makefile_am, makefile_am2, text;
    // Add a '/' to the end of the location if it does not exist
    if (location->get_text().at(location->get_text().length()-1) != '/')
    {
      Glib::ustring str = location->get_text();
      str += "/";
      location->set_text(str);
    }

    // Create the project location, openldev file location and openldev file text
    create_location();
    odp_file_text = create_odp_file_text(odp_file_location);    
    
    // Create directory path
    project_path += location->get_text() + name->get_text() + "/";
    
    // Create copy command, configure.in path, & makefile.am path
    Gtk::Entry *choice = combobox->get_entry();
    Glib::ustring choice_text = choice->get_text();
    
    for (unsigned int i=0; i < tlist->file.size(); i++)
      if (tlist->title.at(i) == choice_text)
      {
        create_project_from_template(tlist->file.at(i));
        i = tlist->file.size();
      }
    
    // Create the project from the template & create the project file
    write_file(odp_file_location, odp_file_text);
    file_location = odp_file_location;
     
    dialog->response(Gtk::RESPONSE_ACCEPT);
    dialog->hide();
  }
  // Throw an error
  else
  {
    error(errortext);
  }
}

// This adds the different types of projects to the combobox
void NewProjectDialog::populate_combobox()
{ combobox->set_popdown_strings(tlist->title);
}

// This allows the main openldev class access to the filename of the project.
Glib::ustring NewProjectDialog::get_filename() 
{ return file_location; 
}
